/*
 *	motor.c
 *	
 * Created 7/20/94 by Chuck McManis
 *
 * This module provides the functions that determine motor speed. They are
 * used to set the motor speed to some value between 0 (off) and 16 (full
 * on.) 
 */

#include <hc11.h>
#include <mboard.h>

unsigned int speed[2];
unsigned char motor_ctl;

/* Table of motor speeds */
static const unsigned int speed_table[17] = {
	0x0000,		/* %0000000000000000	* 0/16 */
	0x0100,		/* %0000000000000001	* 1/16 */
	0x0101,		/* %0000000100000001	* 2/16 */
	0x0842,		/* %0000100001000010	* 3/16 */
	0x1111,		/* %0001000100010001	* 4/16 */
	0x2249,		/* %0010001001001001	* 5/16 */
	0x4929,		/* %0100100100101001	* 6/16 */
	0x52aa,		/* %0101001010101010	* 7/16 */
	0x5555,		/* %0101010101010101	* 8/16 */
	0xaaab,		/* %1010101010101011	* 9/16 */
	0x56ab,		/* %0101011010101011	* 10/16 */
	0xddb6,		/* %1101110110110110	* 11/16 */
	0xeeee,		/* %1110111011101110	* 12/16 */
	0xeeef,		/* %1110111011101111	* 13/16 */
	0xefef,		/* %1110111111101111	* 14/16 */
	0xfeff,		/* %1111111011111111	* 15/16 */
	0xffff		/* %1111111111111111	* 16/16 */
};

void
cmotor(m, s)
	int	m;	/* motor to operate on */
	int	s;	/* speed to set it to. */
{
	unsigned char motor_mask;
	int	i;

	/* These checks take precious space but save on debugging time */
	if ((m > 2) || (m < 1))
		return;
	if ((s < -16) || (s > 16))
		return;

	/* calculate the bits to set in motor_ctl */
	m--;	/* make m = 0 ... 1 */
	i = (m << 1);	/* equals 2 or 0 */
	motor_mask = motor_ctl & ~(0x30 << i);
	if (s == 0) {
		motor_ctl = motor_mask;
		return;
	}
	/* adjust for negative value */
	if (s < 0) {
		motor_mask += (0x10 << i); /* Set the 'reverse' bit */
		s = 0 - s;
	} else if (s > 0)
		motor_mask += (0x20 << i); /* set the 'forward' bit */
	speed[m] = speed_table[s];
	motor_ctl = motor_mask; /* make it so, after speed is set */
	return;
}

/*
 * This is the PWM function for the motor. It uses PORT C.
 * The H-bridge is connected to bits 7,6 and 5,4. The only valid
 * values are one of the pair on and the other off, or both off.
 * 0xc0 = mask for left motor, 0x30 mask for the right motor.
 * motor ctl has the value to send to port C to turn all selected
 * motors *on* and if the bit in the speed variable is off we 
 * disable that.
 * Since it clocks out 1 bit
 * every system tick (1Khz system tick) the frequency at 50% is 500 hz.
 */
void
do_cmotors()
{
	unsigned char 	motor_mask, 
			motors;
	int		i,j;

	if ((motor_ctl & 0xf0) == 0) {
		PORTC = 0; /* eliminates race condition by forcing off */
		return;	/* nothing to do */
	}
	motor_mask = 0x30;	/* Mask information */
	motors = motor_ctl;

	/*
	 * For each motor we check to see if it is enabled in
 	 * the motor_ctl variable, and if it is we then take
	 * its "speed" bitmask and effectively do a rotate right
	 * on it. We preserve the state of the low order bit so
	 * that we can add it back in after the shift (thus making
	 * it a rotation).
	 */
	for (i = 0; i < 2; i++, motor_mask = motor_mask << 2) {
		if ((motor_ctl & motor_mask) == 0)
			continue;
		j = speed[i] & 0x1; /* check the bit */
		speed[i] = speed[i] >> 1; /* shift speed */
		if (j)
			speed[i] |= 0x8000; /* complete the rotate */
		else
			motors &= ~motor_mask; /* turn off the motor */
	}
	PORTC = motors;	/* actually do it */
	return;
}

